import json
import os
import subprocess
import sys
from typing import Dict, List, Optional

from beaupy import select_multiple
from pyfiglet import Figlet
from rich.console import Console
from termcolor import cprint


def install_playwright_browsers():
    console = Console()
    try:
        from playwright.sync_api import sync_playwright

        with sync_playwright() as p:
            browser = p.chromium.launch()
            browser.close()
    except Exception as e:
        print(f"Error: {e}")
        print("Installing Playwright browsers...")
        subprocess.run([sys.executable, "-m", "playwright", "install"], check=True)
        print("Playwright browsers installed successfully.")
    console.clear()


def show_banner():
    try:
        width = os.get_terminal_size()[0]
    except Exception:
        width = 80

    banner = Figlet(font="big", justify="left", width=width)
    cprint(banner.renderText("FollowSphere"), color="blue")


def read_data() -> Optional[Dict]:
    try:
        script_dir = os.path.dirname(os.path.abspath(__file__))
        data_file_path = os.path.join(script_dir, "data.json")
        with open(data_file_path, "r") as file:
            data: Dict = json.load(file)
        return data
    except FileNotFoundError:
        print("File not found")
        exit()


def tags_follower(data: Dict):
    console = Console()
    console.clear()
    show_banner()
    options: List = []
    for k, _v in data.items():
        options.append(k)

    console.print("Which tag collection do you want to follow?", style="red")
    items = select_multiple(sorted(options), tick_character="✓", ticked_indices=None, tick_style="green")
    if items:
        console.print(f"\nYou selected {items}")
        tags = []
        for item in items:
            if data[item]["tags"]:
                for tag in data[item]["tags"]:
                    if tag not in tags:
                        tags.append(tag)
        return tags
    else:
        console.print("You didn't select anything. Terminating the program.")
        exit()



import asyncio
from typing import Dict, List

from playwright.async_api import Browser, ElementHandle, Page, async_playwright

from followsphere.utils import read_data, tags_follower


async def login(page: Page):
    await page.goto("https://www.linkedin.com/login")
    await page.wait_for_selector(".authentication-outlet")


async def follow_hashtags(page: Page, hashtags: List[str]):
    for hashtag in hashtags:
        try:
            await page.goto(f"https://www.linkedin.com/feed/hashtag/{hashtag.lower()}/")
            follow_button: ElementHandle = await page.query_selector("button.follow")
            if follow_button:
                # Check if the button has the 'is-following' class
                class_name = await follow_button.get_attribute("class")
                if "is-following" in class_name:
                    print(f"Already following #{hashtag}.")
                else:
                    # Click the follow button if it's not followed yet
                    await follow_button.click()
                    print(f"Clicked follow for #{hashtag}.")
        except Exception as e:
            print(f"An error occurred for #{hashtag}: {e}")


async def entrypoint(hashtags: List[str]):
    async with async_playwright() as p:
        browser: Browser = await p.chromium.launch(headless=False)  # Set to False to see the browser in action
        page: Page = await browser.new_page()
        await login(page)
        await follow_hashtags(page, hashtags)
        await browser.close()


def execute_linkedin():
    data: Dict = read_data()["LinkedIn"]
    print(data)
    hashtags: List[str] = tags_follower(data)
    asyncio.run(entrypoint(hashtags))



import typer
from beaupy import select
from rich.console import Console

from followsphere.linkedin import execute_linkedin
from followsphere.utils import install_playwright_browsers, show_banner


def main():
    console = Console()
    console.clear()
    install_playwright_browsers()
    show_banner()
    names = [
        "Linkedin",
        "Instagram (Coming soon)",
    ]
    console.print("Choose your platform")
    name: str = select(sorted(names), cursor="🢧", cursor_style="red")
    if name.lower() == "linkedin":
        typer.run(execute_linkedin)
    else:
        typer.echo("\nInvalid platform selected.")


if __name__ == "__main__":
    typer.run(main)
